function [ReshapedData_all, CV_all, data, itsample, iunknown, imeasured] = ...
    FedBatch_MtiFA_SimulationCase1(sampling, error_type, dist_type, n_reps, out)
show=0;
% Possible: integer
if nargin<4; n_rep = 1; n_reps = 1; end
% Possible: 'beta' or 'normal'
if nargin<3; dist_type = 'normal'; end
% Possible: '5', '10', '15' (percentage of error), 'normal' or 'outlier'
if nargin<2; error_type = '5'; end
% Possible: 'hourly' - for every hour - or 'normal' - with night breaks and
% '5' for every five hours evenly distributed
if nargin<1; sampling = 'hourly'; end
% select outlier - 1:length(error)
% 1-C, 2-D, 3-E, 4-Carbon, 5-X, 6-O2, 7-Vol, 8-SumFeed, 9-SumOUR
if nargin<5 && strcmp('outlier', error_type); out = 1; end
    
switch sampling
    case 'normal'
        itsample = [1, 8, 12, 16, 34, 38, 42, 46, 62, 66, 73];
    case 'hourly'
        itsample = 1:2:73; % simulated every 0.5h
    case '5'
        itsample = 1:10:73;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% dcdt order corrected on last part of Fermenter fun not be in the %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%% same order as data %%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[data,Names,Rates,dcdt]=Fermenter(itsample); 
n_rep = n_reps(end);

% corrupt the data with noise
%       C, D, E, O2, Carbon, X, Vol, IntO2, SumFeed, SumOUR
switch error_type
    case '5'
        error = ones(1, size(data, 2)-1) * 0.05;
    case '15'
        error = ones(1, size(data, 2)-1) * 0.15;
    case '10'
        error = ones(1, size(data, 2)-1) * 0.10;
    case 'normal'
        error = [0.05 0.05 0.05 0.05 0.05 0.1 0.02 0.05 0.05];
    case 'outlier'
        error = [0.05 0.05 0.05 0.05 0.05 0.1 0.02 0.05 0.05];
        stdev = 3; % for outlier only
        rng(1234)
        add_out = random(makedist('Normal','mu',0,'sigma',stdev),...
            size(data, 1), 1, n_rep);
end
error(7) = 0.02; % correct volume error
error = repmat(error, size(data, 1), 1);

switch dist_type
    case 'normal'
        pd = makedist('Normal','mu',0,'sigma',1);
    case 'beta'
        pd = makedist('beta','a',2, 'b', 5);
end
data_old = data;
rng(12345)
add = random(pd, size(data, 1), size(data, 2)-1, n_rep);


if strcmp(dist_type, 'beta')
    add = add - mean(mean(mean(add)));
end
%%%%% Poisson only generates integers and positive bias
% if strcmp(dist_type, 'poisson') 
%     signal = random('bino', 1, 0.5, size(data, 1), size(data, 2)-1, n_rep)...
%              * -2 + 1;
%     add = add .* signal;
% end

% Stoichiometric Matrix
%   J1-R4;J2-D_out;J3-E_out;J4-C_out;J5-R1;J6-R2;J7-R3;J8-R5;J9-A_up;J10-R_ft;J11-R_res;J12-O2_up;J13-R_z
S=[ 0     0        0        0       -1     0     0     0     1       0        0         0         0 %A
    0     0        0        0        1    -1     0     0     0       0        0         0         0 %B
   -1     0        0       -1        0     1    -1    -1     0       0        0         0        -1 %C
    3    -1        0        0        0     0     0     0     0       0        0         0         0 %D
    0     0       -1        0        0     0     0     3     0       0        0         0         0 %E
    0     0        0        0        0     0     0     0     0       0       -1         1         0 %O2
    1     0        0        0       -1     2     0     0     0      -1        2         0       -10 %ATP
    0     0        0        0        0     3    10    -2     0       0       -1         0         0 ];%NADH

% Estimation of biomass via Integrated MFA (linear Algebra)
imeasured=[4 2 3 9 12]; % J -> Cout|  Dout| Eout| Aup   | O2up
iunknown=[1 5 6 7 8 10 11 13];

CV_all = zeros(length(itsample), size(S, 2), n_rep);
ReshapedData_all = zeros(73, 5, n_rep);
for f1 = n_reps % Simulate several times
    data(:, 2:end) = data_old(:, 2:end) + add(:,:,f1).*error.*data_old(:, 2:end);
    if strcmp(error_type, 'outlier')
        data(:, out) = data_old(:, out) + add_out(:,:,f1).*error(out).*data_old(:, out);
    end
    if 0==1 % Check effective error
        plot_data(data, Rates, Names);
        d_data = abs(1 - data_old ./ data);
        eff_error = mean(d_data,1,'omitnan');
        bar([eff_error(2:end); error(1, :)]')
        figure
        plot(reshape(data,1,[]),reshape(data_old,1,[]),'o')
    end
    
    % data preparation
    itsample=find(~isnan(data(:,2)));

    % Estimation of the integrated fluxes via linear programming
    imeasured=[4 2 3 9 12]; % This specified which columns in S correspond to the data in DCV
    % J -> Cout | Dout | Eout | Aup | Rz | O2up
    irreversibleFluxes=[]; % Aup 
    ifeed=4;

    %data(:,5)=-data(:,5); % this accounts for the direction of the flux, namely uptake
    S(:,9)=-S(:,9); % this accounts for the direction of the flux, namely uptake
    ReshapedData=[data(:,[2 3 4 5]).*(data(:,8)*ones(1,4)),data(:,10)];
    

    %[CV,Feed]=LinearProgramming_MIFA(ReshapedData(itsample,:),data(itsample,9),ifeed,S,imeasured,irreversibleFluxes,[1:length(itsample)]);
    [CV,Feed,dm]=MtiFA_LP(ReshapedData(itsample,:),data(itsample,9),ifeed,S, imeasured,irreversibleFluxes,[1:length(itsample)]);
    CV_all(:, :, f1) = CV;
    ReshapedData_all(:, :, f1) = ReshapedData;
    
    if show
        figure
        for i=1:length(iunknown)
            subplot(2,4,i);
            plot(data(itsample(2:end),1),diff(CV(:,iunknown(i)))./diff(data(itsample,1))./mean([CV(2:end,13),CV(1:end-1,13)]')','rs','LineWidth',3,'MarkerSize',3);hold on
            plot(data(:,1),Rates(:,iunknown(i)),'bl');axis tight
            xlabel('Time','FontSize',14)
            ylabel(['J_{',int2str(iunknown(i)),'}'],'FontSize',14)
        end
    end
    
    % Calculates the error
    MSE=sum(sum((CV(:,imeasured)-ReshapedData(itsample,:)).^2))/length(itsample)*6;
    % Plot the experimental and predicted values
    if show
        figure;
        for i=1:13
            subplot(4,4,i)
            plot(data(itsample,1),CV(:,i),'rx');hold on;
            j=find(i==imeasured);
            if ~isempty(j)
                plot(data(itsample,1),ReshapedData(itsample,j),'b+');
            end
            axis tight
            xlabel('Time','FontSize',14);
            ylabel(['J_{',int2str(i),'}'],'FontSize',14)
            set(gca,'FontSize',14);
        end
        figure;
        for i=1:length(imeasured)
            subplot(2,3,i);
            plot(CV(:,imeasured(i)),ReshapedData(itsample,i),'rx','LineWidth',2,'MarkerSize',8);
            hold on;
            axis tight;
            h=axis;
            hmin=mean(min([h(1),h(3)]));
            hmax=mean(max([h(2),h(4)]));
            axis square;
            axis([hmin hmax hmin hmax]);
            plot([hmin hmax],[hmin hmax],'k-','LineWidth',2);
            set(gca,'FontSize',14);
            xlabel('Estimated','FontSize',14);
            ylabel('Measured','FontSize',14);
            title(DCV_Name(i),'FontSize',14)
        end
    end
end

end

function [data,Names,Rates,dcdt]=Fermenter(itsample)

if nargin<1
    itsample=[1, 8, 12, 16, 34, 38, 42, 46, 62, 66, 73];
end
cName={'C','D','E','O_2','Carbon','X','Vol','IntO2','SumFeed','SumOUR'};
c0=   [  0,  0,  0, 4e-3,      15,  2,    1,   1e-4,        0,       0,zeros(1,13)]';

[t,c]=ode45(@BioProcessModel,[0:0.5:36],c0);

for i=1:length(c)
    Rates(i,:)=MetabolicNetwork(c(i,:));
end

if 1==0
    figure;
    for i=1:8
        subplot(3,3,i);
        plot(t,c(:,i));
        hold on;
        plot(t(itsample),c(itsample,i),'rx');
        axis tight;
        xlabel('Time');
        ylabel(cName(i))
    end
    figure;
    for i=1:13
        subplot(4,4,i);
        plot(t,Rates(:,i))
        axis tight;
        ylabel(['r_{',int2str(i),'}'])
    end
end

Names={'Time','C','D','E','Carbon','X','O_2','Vol','SumFeed','SumOUR'};
data(1:length(t),1:10)=NaN;
data(:,1)=t;
data(itsample,2:6)=c(itsample,[1 2 3 5 6]);
data(:,7:10)=c(:,[4 7 9 10]);

dcdt=c(:,[11:23]);
end

function [dcdt]=BioProcessModel(t,c)

Cout=c(1);
Dout=c(2);
Eout=c(3);
O2out=c(4);
CarbonOut=c(5);
X=c(6);
Vol=c(7);

% Kinetic Model
J=MetabolicNetwork(c);

% Process Parameters
O2s=5e-3;Oset=.2*O2s; % Assuming that oxygen is controlled at 20% DO
kP=1e5;kI=1e6; % Tuning Parameters PI
RPM=min([max([kP*(Oset-O2out)+kI*c(8),100]),1000]);
kla=0.9*RPM-74.2; % This is a very simplistic kla model taking only the variation in stirrer speed into account
CFeed=200;
if t<8
    uFeed=0;
else
    uFeed=.05;
end

D=uFeed/Vol;

% Material Balance
dcdt(1,:)=J(4)*X-D*Cout; % C
dcdt(2,:)=J(2)*X-D*Dout; % D
dcdt(3,:)=J(3)*X-D*Eout; % E
dcdt(4,:)=-J(12)*X-D*O2out+kla*(O2s-O2out); % O2
dcdt(5,:)=-J(9)*X-D*(CarbonOut-CFeed);      % Carbon
dcdt(6,:)=J(13)*X-D*X;   % X
dcdt(7,:)=uFeed;         % Vol
dcdt(8,:)=max([(Oset-O2out),0]);  % IntO2
dcdt(9,:)=uFeed*CFeed;   % SumFeed
dcdt(10,:)=J(12)*X*Vol;  % SumOUR
dcdt(11:23,:)=J*X*Vol;


end


function J=MetabolicNetwork(c)
    Cout=c(1);
    Dout=c(2);
    Eout=c(3);
    O2out=c(4);
    CarbonOut=c(5);
    X=c(6);
    Vol=c(7);

    J(1:13)=NaN;
    % Kinetic Rates/Fluxes

    kO=.1*4e-3;
    Klim=O2out/(O2out+kO);

    % D - Should depend on C not CarbonOut
    J2maxout=.05;k2=.1;J2maxin=.04;kD=.1;
    J(2)=J2maxout*CarbonOut/(CarbonOut+k2)-J2maxin*Dout/(Dout+kD)*0.02/CarbonOut;
    % E - Should depend on C not CarbonOut
    J3maxout=.07;k3=.1;J3maxin=.06;kE=.1;
    J(3)=J3maxout*CarbonOut/(CarbonOut+k3)-J3maxin*Eout/(Eout+kE)*0.05/CarbonOut;
    % C - Should depend on B not CarbonOut
    J4maxout=.01;k4=.1;J4maxin=.04;kC=.2;
    J(4)=J4maxout*CarbonOut/(CarbonOut+k4)-J4maxin*Cout/(Cout+kC)*0.001/CarbonOut;
    % Carbon (A)
    J9max=.2;kCarbon=.1;
    J(9)=J9max*CarbonOut/(CarbonOut+kCarbon);
    % O2
    J12max=.01;kO2=.07*4e-3;
    J(12)=J12max*O2out/(O2out+kO2);

    % Stoichiometric Matrix
    %   J1-R4;J2-D_out;J3-E_out;J4-C_out;J5-R1;J6-R2;J7-R3;J8-R5;J9-A_up;J10-R_ft;J11-R_res;J12-O2_up;J13-R_z
    S=[ 0     0        0        0       -1     0     0     0     1       0        0         0         0 %A
        0     0        0        0        1    -1     0     0     0       0        0         0         0 %B
        -1    0        0       -1        0     1    -1    -1     0       0        0         0        -1 %C
        3    -1        0        0        0     0     0     0     0       0        0         0         0 %D
        0     0       -1        0        0     0     0     3     0       0        0         0         0 %E
        0     0        0        0        0     0     0     0     0       0       -1         1         0 %O2
        1     0        0        0       -1     2     0     0     0      -1        2         0       -10 %ATP
        0     0        0        0        0     3    10    -2     0       0       -1         0         0 ];%NADH

    % MFA
    J([1 5 6 7 8 10 11 13])=-pinv(S(:,[1 5 6 7 8 10 11 13]))*S(:,[2 3 4 9 12])*J([2 3 4 9 12])';
end


