function [CV_all, FeedEst_all, dm_all, add, noise] = FedBatch_MtiFA_SimulationCase2(sampling, ...
    error_type, dist_type, n_rep, out, is_deter, is_irrev)
%% Check inputs
% Possible: 'hourly' - for every hour - or 'normal' - with night breaks and
% '5' for every five hours evenly distributed
if nargin<1; sampling = 'hourly'; end
% Possible: '5', '10', '15' (percentage of error), 'normal' or 'outlier'
if nargin<2; error_type = '5'; end
% Possible: 'beta' or 'normal'
if nargin<3; dist_type = 'normal'; end
% Possible: integer
if nargin<4; n_rep = 10; end
% Select outlier - 1:length(error)
% 1-C, 2-D, 3-E, 4-Carbon, 5-X, 6-O2, 7-Vol, 8-SumFeed, 9-SumOUR
if nargin<5 && strcmp('outlier', error_type); out = 1; end
% Possible: bool
if nargin<6; is_deter = 1; end
% Possible: bool
if nargin<7; is_irrev = 1; end

%% Set up settings
show = 0;
try load_data = load('reactormodel_case2.mat');
    data = load_data.data;
    t = load_data.t;
catch
    % Data dim:
    c0 = [50, 0, 7, 6, 0.1, 0, 0,    1];
    %     A   H  F  G  I    P  feed, vol-> New
    [t, data] = ReactorModel(c0, show);
    save('reactormodel_case2.mat', 't', 'data', 'c0');
end

switch sampling
    case 'normal'
        iSample = [1, 8, 12, 16, 34, 38, 42, 46, 62, 66, 70, 74, 92, 96, 100];
    case 'hourly'
        iSample = 1:2:100; % simulated every 0.5h
    case '5'
        iSample = 1:10:100;
end

% corrupt the data with noise
%     A   E  L  J  I    G  feed, vol-> Old
switch error_type
    case '5'
        noise = ones(1, size(data, 2)) * 0.05;
    case '15'
        noise = ones(1, size(data, 2)) * 0.15;
    case '10'
        noise = ones(1, size(data, 2)) * 0.10;
    case 'normal'
        noise = [0.01 0.05 0.05 0.05 0.05 0.05 0.5 0.02];
    case 'outlier'
        noise = [0.01 0.05 0.05 0.05 0.05 0.05 0.5 0.02];
        stdev = 3; % For outlier only
        rng(1234)
        add_out = random(makedist('Normal','mu',0,'sigma',stdev),...
            size(data, 1), 1, n_rep);
end
noise(8) = 0.02; % correct volume error
noise = repmat(noise, length(iSample), 1);

switch dist_type
    case 'normal'
        pd = makedist('Normal','mu',0,'sigma',1);
    case 'beta'
        pd = makedist('beta','a',2, 'b', 5);
end
data_old = data;
rng(12345)
add = random(pd, size(data, 1), size(data, 2), n_rep);

if strcmp(dist_type, 'beta')
    add = add - mean(mean(mean(add)));
end

if is_irrev == 1; irrev = [1 2 3 4 5 6]; else; irrev = []; end

if is_deter == 1 % Determined
    imeasured = [1 4 5];
    idata = [1 3 4];
    iFeed = 2;
else             % Overdetermined
    imeasured = [1 2 4 5 6 7];
    idata = [1 2 3 4 5 6];
    iFeed = 3;
end

%    1     2->3  3->2  4     5     6     7
S = [1    -1    -2     0     0     0     0   % B
     0     0     1     0     0    -1    -1   % C
     0     0     0     0     1    -1     0   % D -> E
     0    -1     0     1     0     0     0]; % K -> D
S(:, [2 3]) = S(:, [3 2]);

% This is to be consistent with the flux directions for the reactions
data(:, [1 3 4]) = -data(:, [1 3 4]);

CV_all = zeros(length(iSample), length(idata), n_rep);
FeedEst_all = zeros(1, length(iSample), n_rep);
dm_all = zeros(length(iSample)-1, size(data, 2)-1, n_rep);
for j = 1:n_rep
    % Adding noise to the data
    data_noise = data(iSample, :) + (add(iSample,:,j).*data(iSample, :).*noise);
    
    if strcmp(error_type, 'outlier') && out ~= 0
        data_noise(:, out) = data(iSample, out) +...
            add_out(iSample,:,j).*noise(out).*data_old(iSample, out);
    end
    
    % This multiplies concentration with volume to get masses
    CVexp = data_noise(:, idata).*(data(iSample, 8)*ones(1, length(idata)));
    % This is to be consistent with the flux directions for the reactions
    Feed = -data_noise(:, 7);
    
    [CV, FeedEst, dm] = MtiFA_LP(CVexp, Feed, iFeed, S, imeasured, ...
        irrev, 1:size(data_noise, 1));
    CV_all(:, :, j) = CV;
    FeedEst_all(:, :, j) = FeedEst;
    dm_all(:, :, j) = dm;
    
    
    
    if show == 1
        %%% displaying results
        name = {'A', 'H', 'F', 'G', 'I', 'P', 'Feed', 'Vol'};
        %1-A 2-E 3-L 4-J 5-I 6-G
        figure;
        for i = 1:length(imeasured)
            subplot(2, 4, i);
            plot(t(iSample), CVexp(:, i), '+g');hold on;
            plot(t(iSample), CV(:, i), 'xr');hold on;
            plot(t, data(:, idata(i)).*data(:, 8), 'b');hold on;
            xlabel(name(idata(i)))
        end
        subplot(2, 4, 7);
        plot(t(iSample), -Feed, '+g');hold on;
        plot(t(iSample), -FeedEst, 'xr');hold on;
        plot(t, data(:, 7), 'b');hold on;
        xlabel('Feeding')
        subplot(2, 4, 8);
        plot(t, data(:, 8), 'b');hold on;
        plot(t(iSample), data_noise(:, 8), '+g');hold on;
        
        ireaction = [1 2 4 5 6 7];
        figure;
        for i = 1:6
            subplot(2, 3, i)
            plot(t(iSample), data_noise(:, i).*data(iSample, 8), 'g+');hold on
            if i == 3
                plot(t(iSample(2:end)), data_noise(1, i)...
                    .*data(iSample(1), 8) + cumsum(dm(:, ireaction(i))) +...
                    FeedEst(2:end)', 'rx');
                hold on
            else
                plot(t(iSample(2:end)), data_noise(1, i)...
                    .*data(iSample(1), 8) + cumsum(dm(:, ireaction(i))),...
                    'rx');
                hold on
            end
            plot(t, data(:, i).*data(:, 8),'b');
            hold on
            xlabel(name((i)))
        end
    end
end
end


function [t, data] = ReactorModel(c0, show)

if nargin < 2; show = 0; end
if nargin < 1; c0 = [50, 0, 7, 6, 0.1, 0, 0,    1]; end
%                    A   H  F  G  I    P  feed, vol-> New

[t, data] = ode45(@RedundantBioprocess, 0:1:100, c0);

if show == 1
    figure;
    name = {'A', 'H', 'F', 'G', 'I', 'P', 'Feed', 'Vol'}; % New
    for i = 1:length(name)
        subplot(2, 4, i);
        plot(t, data(:, i));
        hold on;
        xlabel(name(i))
    end
end

end


function dcdt = RedundantBioprocess(t, c)

A   = max([c(1), 0]);
H   = max([c(2), 0]); % E -> H
F   = max([c(3), 0]); % L -> F
G   = max([c(4), 0]); % J -> G
I   = max([c(5), 0]);
P   = max([c(6), 0]);
Vol = c(8);

%    1     2->3  3->2  4     5     6     7
S = [1    -1    -2     0     0     0     0   % B
    0     0     1     0     0    -1    -1   % C
    0     0     0     0     1    -1     0   % D -> E
    0    -1     0     1     0     0     0]; % K -> D
S(:, [2 3]) = S(:, [3 2]);

v(1:7) = 0;
v(1)   = 0.4*A.^2 / (A + 0.1   + H*H*10);        % A
v(4)   = 0.2*F / (F.^3 + 4.5)  * A/(A + 0.0001); % J -> G
v(5)   = 0.1*G.^4 / (G + 10.5) * A.^(1/5)/(A + 0.0001); % I

v([2 3 6 7]) = -pinv(S(:,[2 3 6 7]))*S(:,[1 4 5])*[v(1) v(4) v(5)]';

if t >= 70
    u = 0.05;
else
    u = 0;
end
D = u/Vol;

dcdt(1, :) = -v(1) * I - D*A;       % A
dcdt(2, :) =  v(2) * I - D*H;       % E -> H
dcdt(3, :) = -v(4) * I - D*(F-1.3); % L -> F
dcdt(4, :) = -v(5) * I - D*G;       % J -> G
dcdt(5, :) =  v(6) * I - D*I;       % I == Biomass
dcdt(6, :) =  v(7) * I - D*P;       % G -> P
dcdt(7, :) =  u    * 1.3;
dcdt(8, :) =  u;

end


