function [CVest,Feed,dM]=MtiFA_LP(CV,Feed,ifeed,S,imeasured,irreversibleFluxes,itsample)

% [CVest,Feed,dm]=MtiFA_LP(CV,Feed,ifeed,S,imeasured,irreversibleFluxes,itsample)
%%% INPUTS
    % CV - Concentrations times Volume: This should be total material
    % Feed - This is the accumulated feeding for considered substrates
    % ifeed - specifies to which of the CV the feeding corresponds
    % S - Stoichiometric Matrix
    % imeasured - specified the fluxes of S that are measured, order needs to be in accordance to CV
    % irreversibleFluxes - specifies which fluxes of S are irreversible
    % itsample - specifies for which of the points in CV a soultion should be computed
            
%%% OUTPUTS
    % CVest - Estimated CV
    % Feed - Estimated Feed 
    % dm - Estimated time integrated fluxes for each flux
    
    % nC*ntime,nFeed*ntime;
    nv=size(S,2);
    nC=size(S,1);
    nm=length(imeasured);
    nSample=size(CV,1);
    nitSample=length(itsample);
    nfeed=length(ifeed);
    
    %Scaling of the Concentrations and Feeding to compute the weightings
    %for the objective function
    
    CV_temp =CV;
    CV_temp(CV_temp==0) = 1e-6;
    CV_temp = abs(1./(CV_temp*0.05));
    
    Feed_temp = Feed;
    Feed_temp(Feed_temp==0) = 1e-6;
    Feed_temp = abs(1./(Feed_temp*0.05));
    
    CV_scal =CV_temp;
    Feed_scal=Feed_temp;
    
%   S*Delta_m for all differences between samples
    nAeq=1;
    Aeq=[];
    count=0;
    AllIrrevFlux=[];
    for i=1:nitSample-1
        Aeq(nAeq+count*nC+1:nAeq+count*nC+nC,(itsample(i)-1)*nv+1:itsample(i)*nv)=S;        
        AllIrrevFlux=[AllIrrevFlux,(itsample(i)-1)*nv+irreversibleFluxes];        
        count=count+1;
    end
    nAeq=size(Aeq,1);     
    
    % Aeq dimension 2 is: delta_m * nsample-1 + nmeasured*nsample + nfeed*nsample
    count=0;
    for i=1:nitSample-1
        for j=1:length(imeasured)
            Aeq(nAeq+count*nm+j,(itsample(i)-1)*nv+imeasured(j))=-1; %delta_M
            Aeq(nAeq+count*nm+j,(nitSample-1)*nv+(itsample(i)-1)*nm+j)=-1; %CV_0
            Aeq(nAeq+count*nm+j,(nitSample-1)*nv+itsample(i)*nm+j)=1; %CV_1
        end
        for j=1:nfeed
            Aeq(nAeq+count*nm+ifeed(j),(nitSample-1)*nv+nitSample*nm+(itsample(i)-1)*nfeed+j)=1;
            Aeq(nAeq+count*nm+ifeed(j),(nitSample-1)*nv+nitSample*nm+itsample(i)*nfeed+j)=-1;
        end
        count=count+1;
    end
    nAeq=size(Aeq,1);  
    beq=zeros(1,nAeq);
    

    


    % initilisation of the objective vector
    f=[zeros(1,(nSample-1)*nv+nSample*nm+nSample*nfeed)];
    nf=length(f);

    k=0;
    for i=1:nSample
        %    (CV-CVexp)<=Epsilon
        for j=1:length(imeasured)
            if ~isnan(CV(i,j))
                k=k+1;
                A(k,(nSample-1)*nv+(i-1)*nm+j)=1; % measured CV correspondent
                A(k,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)=-1; %epsilon residual
                b(k)=CV(i,j); %measured CV value
                if ~isempty(CV_scal)
                    f(nf+k)=CV_scal(i,j); % this weights the epsilon residual in the objective function
                end
            end
        end
        %   (Feed-Feedexp <= EpsilonFeed)
        for j=1:nfeed
            if ~isnan(Feed(i,j))
                k=k+1;
                A(k,(nSample-1)*nv+nSample*nm+(i-1)*nfeed+j)=1; % corresponds to measured feed
                A(k,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)=-1; % esplioon for the feed residula
                b(k)=Feed(i,j); % measured accumulated feed value
                if ~isempty(Feed_scal)
                    f(nf+k)=Feed_scal(i,j); % this weights the epsilon of the measured feed value
                end
            end
        end
    end
    A=[A;-abs(A)];%    (CVexp-CV)<=Epsilon & (Feedexp - Feed <= EpsilonFeed)
    b=[b,-b];    
    
    Aeq=[Aeq,zeros(size(Aeq,1),k)]; % this accounts for the extension in the matrix due to the calculated errors
    
    
    % lb - ub 
    lb=ones(1,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)*-inf;
    lb((nSample-1)*nv+nSample*nm+nSample*nfeed+1:(nSample-1)*nv+nSample*nm+nSample*nfeed+k)=0; % Ensuring L1-norm for residulas
    ub=ones(1,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)*inf;
    lb(AllIrrevFlux)=0; % to account for reversibilities
    

    % Solves the optimization problem
    options = optimoptions('linprog','MaxIterations',1e4,'ConstraintTolerance',1e-4,'Display','Iter','Algorithm','dual-simplex');%,'Algorithm','interior-point','Diagnostics','on','FunctionTolerance',.1);%
    x=linprog(f,A,b,Aeq,beq,lb,ub,[],options);

    dM=reshape(x(1:(nSample-1)*nv),nv,(nSample-1))';
    CVest=reshape(x((nSample-1)*nv+1:(nSample-1)*nv+nm*nSample),nm,nSample)';
    Feed=reshape(x((nSample-1)*nv+nSample*nm+1:(nSample-1)*nv+nSample*nm+nfeed*nSample),nfeed,nSample);
    
end