function [CVest,Feed,dm,i0,ObjValue]=tiFBA_MLP(CV,Feed,ifeed,S,imeasured,irreversibleFluxes,itsample,ifluxMaxMin,MaxMinWeight,lb_CV,ub_CV,lb_deltaCV,ub_deltaCV,SparseFBA,IntegerWeight,ilb,iub,Epsilon,M,CV_scal,Feed_scal)

% [CVest,Feed,dm,i0,ObjValue]=tiFBA_MLP(CV,Feed,ifeed,S,imeasured,irreversibleFluxes,itsample,ifluxMaxMin,...
%   MaxMinWeight,lb_CV,ub_CV,lb_deltaCV,ub_deltaCV,SparseFBA,IntegerWeight,ilb,iub,Epsilon,M,CV_scal,Feed_scal)
% tiFBA/tiFVA or sparse tiFBA/tiFVA implementation. Note that in case of
% sparse tiFBA/tiFVA the unknown fluxes are either positive or negative but
% cannot change sign during the experiment.

%%% INPUTS
    % CV - Concentrations times Volume: This should be total material
    % Feed - This is the accumulated feeding for considered substrates
    % ifeed - specifies to which of the CV the feeding corresponds
    % S - Stoichiometric Matrix
    % imeasured - specified the fluxes of S that are measured, order needs to be in accordance to CV
    % irreversibleFluxes - specifies which fluxes of S are irreversible
    % itsample - specifies for which of the points in CV a solution should be computed
    % ifluxMaxMin - specified the flux of S that should be min or max
    % MaxMinWeight - weight attributed to flux that should be max or min    
    % lb_CV - lower bounds on the estimated concentrations*volume dim: nSample x nConcVol
    % ub_CV - upper bounds on the estimated concentrations
    % lb_deltaCV - lower bound on the changes in concentrations dim: nSample x nFluxes 
    % ub_deltaCV - upperound on the changes in concentrations    
    % SparseFBA - 0-time integrated FBA, 1- sparse time integrated FBA     
    % IntegerWeight - weight attributed to binary of the fluxes that forces them to appear/disappear (the greater the fewer fluxes)    
    % ilb - lower bound for the integer (e.g. if it is known that at min there must be 49 fluxes then ilb=49)
    % iub - upper bound for the integer (e.g. the maximum desired number of fluxes is 59 then ilb=59)
    % Epsilon - this is the lowest value dm can take to still be considered active for integer connection
    % M - this is the greatest value dm can take to still be considered active for integer connection
    % CV_scal - to weigh the resdiulas in concentratios
    % Feed_scal - to weigh the residulas in feedings
            
%%% OUTPUTS
    % CVest - Estimated CV
    % Feed - Estimated Feed 
    % dm - Differences in the masses between timepoints
    % i0 - Fluxes that have been determined to be zero under conditions
    % ObjValue - Value returned for the objective


    % nC*ntime,nFeed*ntime;
    nv=size(S,2);
    nC=size(S,1);
    nm=length(imeasured);
    nSample=size(CV,1);
    nitSample=length(itsample);
    nfeed=length(ifeed);
    iunknown=setdiff([1:size(S,2)],imeasured);
    ninteger=length(iunknown);
    
    %  S*dm_{i} for all differences between samples i=1:nSamples -1
    nAeq=1;
    Aeq=[];
    count=0;
    AllIrrevFlux=[];iAllFluxMinMax=[];
    for i=1:nitSample-1
        Aeq(nAeq+count*nC+1:nAeq+count*nC+nC,(itsample(i)-1)*nv+1:itsample(i)*nv)=S;        
        AllIrrevFlux=[AllIrrevFlux,(itsample(i)-1)*nv+irreversibleFluxes];    
        iAllFluxMinMax=[iAllFluxMinMax,(itsample(i)-1)*nv+ifluxMaxMin];% the following section is there to weight the flux(es) that should be maximized/minimized 
        count=count+1;
    end
    nAeq=size(Aeq,1);  
    % Aeq dimension 2 is: delta_m * nsample-1 + nmeasured*nsample + nfeed*nsample
    count=0;
    for i=1:nitSample-1
        for j=1:length(imeasured)
            Aeq(nAeq+count*nm+j,(itsample(i)-1)*nv+imeasured(j))=-1; %delta_M
            Aeq(nAeq+count*nm+j,(nitSample-1)*nv+(itsample(i)-1)*nm+j)=-1; %CV_0
            Aeq(nAeq+count*nm+j,(nitSample-1)*nv+itsample(i)*nm+j)=1; %CV_1
        end
        for j=1:nfeed
            Aeq(nAeq+count*nm+ifeed(j),(nitSample-1)*nv+nitSample*nm+(itsample(i)-1)*nfeed+j)=1;
            Aeq(nAeq+count*nm+ifeed(j),(nitSample-1)*nv+nitSample*nm+itsample(i)*nfeed+j)=-1;
        end
        count=count+1;
    end
    nAeq=size(Aeq,1);  
    beq=zeros(1,nAeq);
    
    % initilisation of the objective vector
    f=[zeros(1,(nSample-1)*nv+nSample*nm+nSample*nfeed)];
    nf=length(f);
    
    
    k=0;
    for i=1:nSample
        %    (CV-CVexp)<=Epsilon
        for j=1:length(imeasured)
            if ~isnan(CV(i,j))
                k=k+1;
                A(k,(nSample-1)*nv+(i-1)*nm+j)=1; % measured CV correspondent
                A(k,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)=-1; %epsilon residual
                b(k)=CV(i,j); %measured CV value
                if ~isempty(CV_scal)
                    f(nf+k)=CV_scal(i,j); % this weights the epsilon residual in the objective function
                end
            end
        end
        %   (Feed-Feedexp <= EpsilonFeed)
        for j=1:nfeed
            if ~isnan(Feed(i,j))
                k=k+1;
                A(k,(nSample-1)*nv+nSample*nm+(i-1)*nfeed+j)=1; % corresponds to measured feed
                A(k,(nSample-1)*nv+nSample*nm+nSample*nfeed+k)=-1; % esplioon for the feed residula
                b(k)=Feed(i,j); % measured accumulated feed value
                if ~isempty(Feed_scal)
                    f(nf+k)=Feed_scal(i,j); % this weights the epsilon of the measured feed value
                end
            end
        end
    end
    A=[A;-abs(A)];%    (CVexp-CV)<=Epsilon & (Feedexp - Feed <= EpsilonFeed)
    b=[b,-b];    
    
    Aeq=[Aeq,zeros(size(Aeq,1),k)]; % this accounts for the extension in the matrix due to the calculated errors
    
    f(iAllFluxMinMax)=MaxMinWeight; % this means all flux differences are equally weighted in the min/max of the fluxes, one could however also weight specific differences more
    
    % this corrects the bounds for the fluxes to adhere to reversibilities
    for i=1:nSample-1   % this ensures irreversibility between two neighbouring samples
        for j=1:length(irreversibleFluxes)
            if ~isempty(lb_deltaCV)
                if lb_deltaCV(i,irreversibleFluxes(j))<0
                    disp(' ');
                    disp('Some of the lower bounds on DeltaCV violate the irreversibilities. The violating bounds are reset to 0.');
                    irreversibleFluxes(j)
                    lb_deltaCV(i,irreversibleFluxes(j))=0;
                end
            end
        end
    end
    % lb - ub - These are lower and upper bounds for the material (i.e. Conc*Vol)
    lb=ones(1,(nSample-1)*nv+nSample*(nm+nfeed)+k)*-999;
    lb(AllIrrevFlux)=0; % to account for reversibilities    
    lb((nSample-1)*nv+nSample*(nm+nfeed)+1:(nSample-1)*nv+nSample*(nm+nfeed)+k)=0;
    ub=ones(1,(nSample-1)*nv+nSample*(nm+nfeed)+k)*999;
     % The following sets the upper and lower bounds on the chnages in concentrations
    if ~isempty(lb_deltaCV)
        lb(1:(nSample-1)*nv)=reshape(lb_deltaCV',1,nv*(nSample-1));
        ub(1:(nSample-1)*nv)=reshape(ub_deltaCV',1,nv*(nSample-1));
    end        
    if ~isempty(lb_CV)
        for i=1:nSample
            for j=1:nm
               lb((nSample-1)*nv+(i-1)*nm+j)=lb_CV(i,j);
               ub((nSample-1)*nv+(i-1)*nm+j)=ub_CV(i,j);
            end
        end
    end
    
    if SparseFBA==0
        % tiFBA solution     
        options = optimoptions('linprog','ConstraintTolerance',1e-6,'Display','Iter');%,'Algorithm','interior-point','Diagnostics','on','FunctionTolerance',.1);%
        x=linprog(f,A,b,Aeq,beq,lb,ub,[],options);
        dm=reshape(x(1:nv*(nSample-1)),nv,nSample-1)';
        CVest=reshape(x(nv*(nSample-1)+1:nv*(nSample-1)+nm*nSample),nm,nSample)';
        Feed=reshape(x(nv*(nSample-1)+nm*nSample+1:nv*(nSample-1)+nm*nSample+nfeed*nSample),nfeed,nSample);
    else
        %the following is to establish the integer variables and link them to the estimated fluxes
        nA=size(A,1);
        A=[A,zeros(nA,ninteger+length(intersect(iunknown,irreversibleFluxes)))];
        nAeq=size(Aeq,1);
        Aeq=[Aeq,zeros(nAeq,ninteger+length(intersect(iunknown,irreversibleFluxes)))];
        iq=0;
        q=0;
        %Epsilon=1e-4;M=1e4;
        [~,iunknownIrreversible,~]=intersect(iunknown,irreversibleFluxes);
        qlast=0;
        for i=1:nSample-1
            for j=1:ninteger
                iq=iq+1;
                A(nA+iq,(i-1)*nv+iunknown(j))=-1; 
                A(nA+iq,(nSample-1)*nv+nSample*(nm+nfeed)+k+j)=Epsilon;
                b(nA+iq)=0;
                iq=iq+1;
                A(nA+iq,(i-1)*nv+iunknown(j))=1; 
                A(nA+iq,(nSample-1)*nv+nSample*(nm+nfeed)+k+j)=-M; 
                b(nA+iq)=0;
                if ~isempty(find(iunknown(j)==irreversibleFluxes))
                    q=find(iunknown(iunknownIrreversible)==iunknown(j)); % This is for the reversible fluxes the counter
                    A(nA+iq-1,(nSample-1)*nv+nSample*(nm+nfeed)+k+ninteger+q)=-M; % Reversible and Irreversible can be acctive at the same time, bu this will cost additionally
                    A(nA+iq,(nSample-1)*nv+nSample*(nm+nfeed)+k+ninteger+q)=Epsilon;
                    if qlast~=q
                        iq=iq+1;
                        A(nA+iq,(nSample-1)*nv+nSample*(nm+nfeed)+k+j)=1;
                        A(nA+iq,(nSample-1)*nv+nSample*(nm+nfeed)+k+ninteger+q)=1;
                        b(nA+iq)=1;
                        qlast=q;
                    end
                end            
            end
        end
        Int=(nSample-1)*nv+nSample*(nm+nfeed)+k+1:(nSample-1)*nv+nSample*(nm+nfeed)+k+ninteger+q;
        f=[f,zeros(1,ninteger+q)];
        f(Int)=IntegerWeight;
        lb(Int)=0;
        ub(Int)=1;

        nA=size(A,1);
        if ~isempty(ilb)
            A(nA+1,Int)=-1;
            b(nA+1)=-ilb;
        end

        nA=size(A,1);
        if ~isempty(iub)
            A(nA+1,Int)=1;
            b(nA+1)=iub;
        end

        % Solves the optimization problem
        options = optimoptions('intlinprog','ConstraintTolerance',1e-8,'Display','Iter','IntegerTolerance',1e-6);%,'RootLPAlgorithm','primal-simplex','CutGeneration','intermediate');%'NodeSelection','minobj','Heuristics','rins');%,'Algorithm','interior-point','Diagnostics','on','FunctionTolerance',.1);%
        x=intlinprog(f,Int,A,b,Aeq,beq,lb,ub,options);

        dm=reshape(x(1:nv*(nSample-1)),nv,nSample-1)';
        CVest=reshape(x(nv*(nSample-1)+1:nv*(nSample-1)+nm*nSample),nm,nSample)';
        Feed=reshape(x(nv*(nSample-1)+nm*nSample+1:nv*(nSample-1)+nm*nSample+nfeed*nSample),nfeed,nSample);
        nIntStart=nv*(nSample-1)+nm*nSample+nfeed*nSample+k;
        i0=setdiff(iunknown,sort([iunknown(find(round(x(nIntStart+1:nIntStart+ninteger)))),iunknown(iunknownIrreversible(find(round(x(nIntStart+ninteger+1:nIntStart+ninteger+q)))))]));
        ObjValue=f*x;
    end

end